#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Script to check recently uploaded files. This script checks if a file
description is present and if there are other problems in the image's description.

This script will have to be configured for each language. Please submit
translations as addition to the pywikipediabot framework.

Everything that needs customisation is indicated by comments.

This script understands the following command-line arguments:

-limit              The number of images to check (default: 80)

-commons            The Bot will check if an image on Commons has the same name
                    and if true it report the image.

-duplicates[:#]     Checking if the image has duplicates (if arg, set how many
                    rollback wait before reporting the image in the report
                    instead of tag the image) default: 1 rollback.

-duplicatesreport   Report the duplicates in a log *AND* put the template in
                    the images.

-sendemail          Send an email after tagging.

-break              To break the bot after the first check (default: recursive)

-time[:#]           Time in seconds between repeat runs (default: 30)

-wait[:#]           Wait x second before check the images (default: 0)

-skip[:#]           The bot skip the first [:#] images (default: 0)

-start[:#]          Use allpages() as generator
                    (it starts already form File:[:#])

-cat[:#]            Use a category as generator

-regex[:#]          Use regex, must be used with -url or -page

-page[:#]           Define the name of the wikipage where are the images

-url[:#]            Define the url where are the images

-untagged[:#]       Use daniel's tool as generator:
                    http://toolserver.org/~daniel/WikiSense/UntaggedImages.php

-nologerror         If given, this option will disable the error that is risen
                    when the log is full.

---- Instructions for the real-time settings  ----
* For every new block you have to add:

<------- ------->

In this way the Bot can understand where the block starts in order to take the
right parameter.

* Name=     Set the name of the block
* Find=     Use it to define what search in the text of the image's description,
            while
  Findonly= search only if the exactly text that you give is in the image's
            description.
* Summary=  That's the summary that the bot will use when it will notify the
            problem.
* Head=     That's the incipit that the bot will use for the message.
* Text=     This is the template that the bot will use when it will report the
            image's problem.

---- Known issues/FIXMEs: ----
* Clean the code, some passages are pretty difficult to understand if you're not the coder.
* Add the "catch the language" function for commons.
* Fix and reorganise the new documentation
* Add a report for the image tagged.

"""

#
# (C) Kyle/Orgullomoore, 2006-2007 (newimage.py)
# (C) Siebrand Mazeland, 2007-2010
# (C) Filnik, 2007-2011
# (C) Pywikipedia team, 2007-2011
#
# Distributed under the terms of the MIT license.
#
__version__ = '$Id: checkimages.py 9771 2011-11-16 17:37:38Z filnik $'
#

import re, time, urllib, urllib2, os, locale, sys, datetime
import wikipedia as pywikibot
import config, pagegenerators, catlib, query, userlib
import checkimages

locale.setlocale(locale.LC_ALL, '')

###############################################################################
# <--------------------------- Change only below! --------------------------->#
###############################################################################

# NOTE: in the messages used by the Bot if you put __botnick__ in the text, it
# will automatically replaced with the bot's nickname.

# That's what you want that will be added. (i.e. the {{no source}} with the
# right day/month/year )
n_txt = {
    'commons':u'{{subst:nld}}',
    'ar'     :u'{{subst:لم}}',
    'de'     :u'{{Benutzer:ABF/D|~~~~}} {{Dateiüberprüfung/benachrichtigt (Kategorie)|{{subst:LOCALYEAR}}|{{subst:LOCALMONTH}}|{{subst:LOCALDAY}}}} {{Dateiüberprüfung/benachrichtigt (Text)|Lizenz|||||}} --This was added by ~~~~-- ',
    'en'     :u'{{subst:nld}}',
    'fa'     :u'{{جا:حق تکثیر تصویر نامعلوم}}',
    'fr'     :u'{{subst:lid}}',
    'ga'     :u'{{subst:Ceadúnas de dhíth}}',
    'hu'     :u'{{nincslicenc|~~~~~}}',
    'it'     :u'{{subst:unverdata}}',
    'ja'     :u'{{subst:Nld}}',
    'ko'     :u'{{subst:nld}}',
    'ta'     :u'{{subst:nld}}',
    'zh'     :u'{{subst:No license/auto}}',
}

# Text that the bot will try to see if there's already or not. If there's a
# {{ I'll use a regex to make a better check.
# This will work so:
# '{{nld' --> '\{\{(?:template:|)no[ _]license ?(?:\||\n|\}) ?' (case
# insensitive).
# If there's not a {{ it will work as usual (if x in Text)
txt_find =  {
    'commons':[u'{{no license', u'{{no license/en', u'{{nld', u'{{no permission', u'{{no permission since'],
    'ar':[u'{{لت', u'{{لا ترخيص'],
    'de':[u'{{DÜP', u'{{Dateiüberprüfung'],
    'en':[u'{{nld', u'{{no license'],
    'fa':[u'{{حق تکثیر تصویر نامعلوم۲'],
    'ga':[u'{{Ceadúnas de dhíth', u'{{Ceadúnas de dhíth'],
    'hu':[u'{{nincsforrás',u'{{nincslicenc'],
    'it':[u'{{unverdata', u'{{unverified'],
    'ja':[u'{{no source', u'{{unknown', u'{{non free', u'<!--削除についての議論が終了するまで',],
    'ta':[u'{{no source', u'{{nld', u'{{no license'],
    'ko':[u'{{출처 없음', u'{{라이선스 없음',u'{{Unknown',],
    'zh':[u'{{no source', u'{{unknown', u'{{No license',],
}

# Summary for when the will add the no source
comm = {
    'ar'     :u'بوت: التعليم على ملف مرفوع حديثا غير موسوم',
    'commons':u'Bot: Marking newly uploaded untagged file',
    'de'     :u'Bot: Markierung als Bild ohne Lizenz',
    'en'     :u'Bot: Marking newly uploaded untagged file',
    'fa'     :u'ربات: حق تکثیر تصویر تازه بارگذاری شده نامعلوم است.',
    'ga'     :u'Róbó: Ag márcáil comhad nua-uaslódáilte gan ceadúnas',
    'hu'     :u'Robot: Frissen feltöltött licencsablon nélküli fájl megjelölése',
    'it'     :u"Bot: Aggiungo unverified",
    'ja'     :u'ロボットによる:著作権情報なしの画像をタグ',
    'ko'     :u'로봇:라이선스 없음',
    'ta'     :u'தானியங்கி:காப்புரிமை வழங்கப்படா படிமத்தை சுட்டுதல்',
    'zh'     :u'機器人:標示新上傳且未包含必要資訊的檔案',
}

# When the Bot find that the usertalk is empty is not pretty to put only the
# no source without the welcome, isn't it?
empty = {
    'commons':u'{{subst:welcome}}\n~~~~\n',
    'ar'     :u'{{ترحيب}}\n~~~~\n',
    'de'     :u'{{subst:willkommen}} ~~~~',
    'en'     :u'{{welcome}}\n~~~~\n',
    'fa'     :u'{{جا:خوشامدید|%s}}',
    'fr'     :u'{{Bienvenue nouveau\n~~~~\n',
    'ga'     :u'{{subst:Fáilte}} - ~~~~\n',
    'hu'     :u'{{subst:Üdvözlet|~~~~}}\n',
    'it'     :u'<!-- inizio template di benvenuto -->\n{{subst:Benvebot}}\n~~~~\n<!-- fine template di benvenuto -->',
    'ja'     :u'{{subst:Welcome/intro}}\n{{subst:welcome|--~~~~}}\n',
    'ko'     :u'{{환영}}--~~~~\n',
    'ta'     :u'{{welcome}}\n~~~~\n',
    'zh'     :u'{{subst:welcome|sign=~~~~}}',
}

# Summary that the bot use when it notify the problem with the image's license
comm2 = {
    'ar'     :u'بوت: طلب معلومات المصدر.',
    'commons':u'Bot: Requesting source information.',
    'de'     :u'Bot:Notify User',
    'en'     :u'Robot: Requesting source information.',
    'fa'     :u'ربات: درخواست منبع تصویر',
    'ga'     :u'Róbó: Ag iarraidh eolais foinse.',
    'it'     :u"Bot: Notifico l'unverified",
    'hu'     :u'Robot: Forrásinformáció kérése',
    'ja'     :u'ロボットによる:著作権情報明記のお願い',
    'ko'     :u'로봇:라이선스 정보 요청',
    'ta'     :u'தானியங்கி:மூலம் வழங்கப்படா படிமத்தை சுட்டுதல்',
    'zh'     :u'機器人：告知用戶',
}

# if the file has an unknown extension it will be tagged with this template.
# In reality, there aren't unknown extension, they are only not allowed...
delete_immediately = {
    'commons':u"{{speedy|The file has .%s as extension. Is it ok? Please check.}}",
    'ar'     :u"{{شطب|الملف له .%s كامتداد.}}",
    'en'     :u"{{db-meta|The file has .%s as extension.}}",
    'fa'     :u"{{حذف سریع|تصویر %s اضافی است.}}",
    'ga'     :u"{{scrios|Tá iarmhír .%s ar an comhad seo.}}",
    'hu'     :u'{{azonnali|A fájlnak .%s a kiterjesztése}}',
    'it'     :u'{{cancella subito|motivo=Il file ha come estensione ".%s"}}',
    'ja'     :u'{{db|知らないファイルフォーマット %s}}',
    'ko'     :u'{{delete|잘못된 파일 형식 (.%s)}}',
    'ta'     :u'{{delete|இந்தக் கோப்பு .%s என்றக் கோப்பு நீட்சியைக் கொண்டுள்ளது.}}',
    'zh'     :u'{{delete|未知檔案格式%s}}',
}

# The header of the Unknown extension's message.
delete_immediately_head = {
    'commons':u"\n== Unknown extension! ==\n",
    'ar'     :u"\n== امتداد غير معروف! ==\n",
    'en'     :u"\n== Unknown extension! ==\n",
    'fa'     :u"\n==بارگذاری تصاویر موجود در انبار==\n",
    'ga'     :u"\n== Iarmhír neamhaithnid! ==\n",
    'fr'     :u'\n== Extension inconnue ==\n',
    'hu'     :u'\n== Ismeretlen kiterjesztésű fájl ==\n',
    'it'     :u'\n\n== File non specificato ==\n',
    'ko'     :u'\n== 잘못된 파일 형식 ==\n',
    'ta'     :u'\n== இனங்காணப்படாத கோப்பு நீட்சி! ==\n',
    'zh'     :u'\n==您上載的檔案格式可能有誤==\n',
}

# Text that will be add if the bot find a unknown extension.
delete_immediately_notification = {
    'ar'     :u'الملف [[:File:%s]] يبدو أن امتداده خاطيء, من فضلك تحقق. ~~~~',
    'commons':u'The [[:File:%s]] file seems to have a wrong extension, please check. ~~~~',
    'en'     :u'The [[:File:%s]] file seems to have a wrong extension, please check. ~~~~',
    'fa'     :u'به نظر می‌آید تصویر [[:تصویر:%s]] مسیر نادرستی داشته باشد لطفا بررسی کنید.~~~~',
    'ga'     :u'Tá iarmhír mícheart ar an comhad [[:File:%s]], scrúdaigh le d\'thoil. ~~~~',
    'fr'     :u'Le fichier [[:File:%s]] semble avoir une mauvaise extension, veuillez vérifier. ~~~~',
    'hu'     :u'A [[:Kép:%s]] fájlnak rossz a kiterjesztése, kérlek ellenőrízd. ~~~~',
    'it'     :u'{{subst:Progetto:Coordinamento/Immagini/Bot/Messaggi/Ext|%s|__botnick__}} --~~~~',
    'ko'     :u'[[:그림:%s]]의 파일 형식이 잘못되었습니다. 확인 바랍니다.--~~~~',
    'ta'     :u'[[:படிமம்:%s]] இனங்காணப்படாத கோப்பு நீட்சியை கொண்டுள்ளது தயவு செய்து ஒரு முறை சரி பார்க்கவும் ~~~~',
    'zh'    :u'您好，你上傳的[[:File:%s]]無法被識別，請檢查您的檔案，謝謝。--~~~~',
}

# Summary of the delete immediately.
# (e.g: Adding {{db-meta|The file has .%s as extension.}})
del_comm = {
    'ar'     :u'بوت: إضافة %s',
    'commons':u'Bot: Adding %s',
    'en'     :u'Bot: Adding %s',
    'fa'     :u'ربات: اضافه کردن %s',
    'ga'     :u'Róbó: Cuir %s leis',
    'fr'     :u'Robot : Ajouté %s',
    'hu'     :u'Robot:"%s" hozzáadása',
    'it'     :u'Bot: Aggiungo %s',
    'ja'     :u'ロボットによる: 追加 %s',
    'ko'     :u'로봇 : %s 추가',
    'ta'     :u'Bot: Adding %s',
    'zh'     :u'機器人: 正在新增 %s',
}

# This is the most important header, because it will be used a lot. That's the
# header that the bot will add if the image hasn't the license.
nothing_head = {
    'ar'     :u"\n== صورة بدون ترخيص ==\n",
    'commons':u"",# Nothing, the template has already the header inside.
    'de'     :u"\n== Bild ohne Lizenz ==\n",
    'en'     :u"\n== Image without license ==\n",
    'ga'     :u"\n== Comhad gan ceadúnas ==\n",
    'fr'     :u"\n== Fichier sans licence ==\n",
    'hu'     :u"\n== Licenc nélküli kép ==\n",
    'it'     :u"\n\n== File senza licenza ==\n",
    'ja'     :u'',
    'ko'     :u'',
    'fa'     :u'',
    'ta'     :u'',
    'zh'     :u'',
    }
# That's the text that the bot will add if it doesn't find the license.
# Note: every __botnick__ will be repleaced with your bot's nickname (feel free not to use if you don't need it)
nothing_notification = {
    'commons':u"\n{{subst:User:Filnik/untagged|File:%s}}\n\n''This message was '''added automatically by " + \
               "__botnick__''', if you need some help about it, please read the text above again and follow the links in it," + \
            "if you still need help ask at the [[File:Human-help-browser.svg|18px|link=Commons:Help desk|?]] '''[[Commons:Help desk|->]]" + \
            "[[Commons:Help desk]]''' in any language you like to use.'' --__botnick__ ~~~~~""",
    'ar'     :u"{{subst:مصدر الصورة|File:%s}} --~~~~",
    'de'     :u'\n{{subst:Benutzer:ABF/D2|%s}} ~~~~ ',
    'en'     :u"{{subst:image source|File:%s}} --~~~~",
    'fa'     :u"{{جا:اخطار نگاره|%s}}",
    'ga'     :u"{{subst:Foinse na híomhá|File:%s}} --~~~~",
    'hu'     :u"{{subst:adjforrást|Kép:%s}} \n Ezt az üzenetet ~~~ automatikusan helyezte el a vitalapodon, kérdéseddel fordulj a gazdájához, vagy a [[WP:KF|Kocsmafalhoz]]. --~~~~",
    'it'     :u"{{subst:Progetto:Coordinamento/Immagini/Bot/Messaggi/Senza licenza|%s|__botnick__}} --~~~~",
    'ja'     :u"\n{{subst:Image copyright|File:%s}}--~~~~",
    'ko'     :u'\n{{subst:User:Kwjbot IV/untagged|%s}} --~~~~',
    'ta'     :u'\n{{subst:Di-no license-notice|படிமம்:%s}} ~~~~ ',
    'zh'     :u'\n{{subst:Uploadvionotice|File:%s}} ~~~~ ',
}

# This is a list of what bots used this script in your project.
# NOTE: YOUR Botnick is automatically added. It's not required to add it twice.
bot_list = {
    'commons':[u'Siebot', u'CommonsDelinker', u'Filbot', u'John Bot', u'Sz-iwbot', u'ABFbot'],
    'de'     :[u'ABFbot'],
    'en'     :[u'OrphanBot'],
    'fa'     :[u'Amirobot'],
    'ga'     :[u'AllieBot'],
    'it'     :[u'Filbot', u'Nikbot', u'.snoopyBot.'],
    'ja'     :[u'Alexbot'],
    'ko'     :[u'Kwjbot IV'],
    'ta'     :[u'TrengarasuBOT'],
    'zh'     :[u'Alexbot'],
}

# The message that the bot will add the second time that find another license
# problem.
second_message_without_license = {
    '_default':None,
    'hu':u'\nSzia! Úgy tűnik a [[:Kép:%s]] képpel is hasonló a probléma, mint az előbbivel. Kérlek olvasd el a [[WP:KÉPLIC|feltölthető képek]]ről szóló oldalunk, és segítségért fordulj a [[WP:KF-JO|Jogi kocsmafalhoz]]. Köszönöm --~~~~',
    'it':u':{{subst:Progetto:Coordinamento/Immagini/Bot/Messaggi/Senza licenza2|%s|__botnick__}} --~~~~',
}

# You can add some settings to wikipedia. In this way, you can change them
# without touching the code. That's useful if you are running the bot on
# Toolserver.
page_with_settings = {
    '_default':None,
    'commons':u'User:Filbot/Settings',
    'it':u'Progetto:Coordinamento/Immagini/Bot/Settings#Settings',
    'zh':u"User:Alexbot/cisettings#Settings",
}

# The bot can report some images (like the images that have the same name of an
# image on commons) This is the page where the bot will store them.
report_page = {
    'commons':u'User:Filbot/Report',
    'de'     :u'Benutzer:ABFbot/Report',
    'en'     :u'User:Filnik/Report',
    'fa'     :u'کاربر:Amirobot/گزارش تصویر',
    'ga'     :u'User:AllieBot/ReportImages',
    'hu'     :u'User:Bdamokos/Report',
    'it'     :u'Progetto:Coordinamento/Immagini/Bot/Report',
    'ja'     :u'User:Alexbot/report',
    'ko'     :u'User:Kwjbot IV/Report',
    'ta'     :u'User:Trengarasu/commonsimages',
    'zh'     :u'User:Alexsh/checkimagereport',
}

# Adding the date after the signature.
timeselected = u' ~~~~~'

# The text added in the report
report_text = {
    'commons':u"\n*[[:File:%s]] " + timeselected,
    'ar':u"\n*[[:ملف:%s]] " + timeselected,
    'de':u"\n*[[:Bild:%s]] " + timeselected,
    'en':u"\n*[[:File:%s]] " + timeselected,
    'fa':u"n*[[:پرونده:%s]] "+ timeselected,
    'ga':u"\n*[[:File:%s]] " + timeselected,
    'hu':u"\n*[[:Kép:%s]] " + timeselected,
    'it':u"\n*[[:File:%s]] " + timeselected,
    'ja':u"\n*[[:File:%s]] " + timeselected,
    'ko':u"\n*[[:그림:%s]] " + timeselected,
    'ta':u"\n*[[:படிமம்:%s]] " + timeselected,
    'zh':u"\n*[[:File:%s]] " + timeselected,
}

# The summary of the report
comm10 = {
    'commons':u'Bot: Updating the log',
    'ar'     :u'بوت: تحديث السجل',
    'de'     :u'Bot: schreibe Log',
    'en'     :u'Bot: Updating the log',
    'fa'     :u'ربات: به‌روزرسانی سیاهه',
    'fr'     :u'Robot: Mise à jour du journal',
    'ga'     :u'Róbó: Log a thabhairt suas chun dáta',
    'hu'     :u'Robot: A napló frissítése',
    'it'     :u'Bot: Aggiorno il log',
    'ja'     :u'ロボットによる:更新',
    'ko'     :u'로봇:로그 업데이트',
    'ta'     :u'தானியங்கி:பட்டியலை இற்றைப்படுத்தல்',
    'zh'     :u'機器人:更新記錄',
}

# If a template isn't a license but it's included on a lot of images, that can
# be skipped to analyze the image without taking care of it. (the template must
# be in a list)
# Warning:   Don't add template like "en, de, it" because they are already in
#            (added in the code, below
# Warning 2: The bot will use regex, make the names compatible, please (don't
#            add "Template:" or {{because they are already put in the regex).
# Warning 3: the part that use this regex is case-insensitive (just to let you
#            know..)
HiddenTemplate = {
    'commons':[u'Template:Information'], # Put the other in the page on the project defined below
    'ar':[u'Template:معلومات'],
    'de':[u'Template:Information'],
    'en':[u'Template:Information'],
    'fa':[u'الگو:اطلاعات'],
    'fr':[u'Template:Information'],
    'ga':[u'Template:Information'],
    'hu':[u'Template:Információ', u'Template:Enwiki', u'Template:Azonnali'],
    'it':[u'Template:EDP', u'Template:Informazioni file', u'Template:Information', u'Template:Trademark', u'Template:Permissionotrs'], # Put the other in the page on the project defined below
    'ja':[u'Template:Information'],
    'ko':[u'Template:그림 정보'],
    'ta':[u'Template:Information'],
    'zh':[u'Template:Information'],
}

# A page where there's a list of template to skip.
PageWithHiddenTemplates = {
    '_default':None,
    'commons': u'User:Filbot/White_templates#White_templates',
    'it':u'Progetto:Coordinamento/Immagini/Bot/WhiteTemplates',
    'ko': u'User:Kwjbot_IV/whitetemplates/list',
}

# A page where there's a list of template to consider as licenses.
PageWithAllowedTemplates = {
    '_default':None,
    'commons': u'User:Filbot/Allowed templates',
    'it':u'Progetto:Coordinamento/Immagini/Bot/AllowedTemplates',
    'ko':u'User:Kwjbot_IV/AllowedTemplates',
}

# Template added when the bot finds only an hidden template and nothing else.
# Note: every __botnick__ will be repleaced with your bot's nickname (feel free not to use if you don't need it)
HiddenTemplateNotification = {
    '_default':None,
    'commons': u"""\n{{subst:User:Filnik/whitetemplate|File:%s}}\n\n''This message was added automatically by __botnick__, if you need some help about it please read the text above again and follow the links in it, if you still need help ask at the [[File:Human-help-browser.svg|18px|link=Commons:Help desk|?]] '''[[Commons:Help desk|→]] [[Commons:Help desk]]''' in any language you like to use.'' --__botnick__ ~~~~~""",
    'it'     : u"{{subst:Progetto:Coordinamento/Immagini/Bot/Messaggi/Template_insufficiente|%s|__botnick__}} --~~~~",
    'ko'     : u"\n{{subst:User:Kwj2772/whitetemplates|%s}} --~~~~",
}

# In this part there are the parameters for the dupe images.

# Put here the template that you want to put in the image to warn that it's a dupe
# put __image__ if you want only one image, __images__ if you want the whole list
duplicatesText = {
    '_default':None,
    'commons': u'\n{{Dupe|__image__}}',
    'it'     : u'\n{{Progetto:Coordinamento/Immagini/Bot/Template duplicati|__images__}}',
}

# Head of the message given to the author
duplicate_user_talk_head = {
    '_default':None,
    'it'     : u'\n\n== File doppio ==\n',
}

# Message to put in the talk
duplicates_user_talk_text = {
    '_default':None,
    'commons': u'{{subst:User:Filnik/duplicates|File:%s|File:%s}}', # FIXME: it doesn't exist
    'it'     : u"{{subst:Progetto:Coordinamento/Immagini/Bot/Messaggi/Duplicati|%s|%s|__botnick__}} --~~~~",
}

# Comment used by the bot while it reports the problem in the uploader's talk
duplicates_comment_talk = {
    '_default':None,
    'commons': u'Bot: Dupe file found',
    'ar'     : u'بوت: ملف مكرر تم العثور عليه',
    'it'     : u"Bot: Notifico il file doppio trovato",
}

# Comment used by the bot while it reports the problem in the image
duplicates_comment_image = {
    '_default':None,
    'commons': u'Bot: Tagging dupe file',
    'ar'     : u'بوت: وسم ملف مكرر',
    'it'     : u'Bot: File doppio, da cancellare',
}

# Regex to detect the template put in the image's decription to find the dupe
duplicatesRegex = {
    '_default':None,
    'commons': r'\{\{(?:[Tt]emplate:|)(?:[Dd]up(?:licat|)e|[Bb]ad[ _][Nn]ame)[|}]',
    'it'     : r'\{\{(?:[Tt]emplate:|)[Pp]rogetto:[Cc]oordinamento/Immagini/Bot/Template duplicati[|}]',
}

# Category with the licenses and / or with subcategories with the other licenses.
category_with_licenses = {
    'commons': 'Category:License tags',
    'ar': 'تصنيف:قوالب حقوق الصور',
    'en': 'Category:Wikipedia image copyright templates',
    'fa': u'رده:برچسب‌های حق تکثیر نگاره',
    'ga': 'Catagóir:Clibeanna cóipchirt d\'íomhánna',
    'it': 'Categoria:Template Licenze copyright',
    'ja': 'Category:画像の著作権表示テンプレート',
    'ko': '분류:위키백과 그림 저작권 틀',
    'ta': 'Category:காப்புரிமை வார்ப்புருக்கள்',
    'zh': 'Category:版權申告模板',
}

## Put None if you don't use this option or simply add nothing if en
## is still None.
# Page where is stored the message to send as email to the users
emailPageWithText = {
    '_default':None,
    'de':'Benutzer:ABF/D3',
}

# Title of the email
emailSubject = {
    '_default':None,
    'de':'Problemen mit Deinem Bild auf der Deutschen Wikipedia',
}

# Seems that uploaderBots aren't interested to get messages regarding the
# files that they upload.. strange, uh?
# Format: [[user,regex], [user,regex]...] the regex is needed to match the user where to send the warning-msg
uploadBots = {
        '_default':None,
        'commons':[['File Upload Bot (Magnus Manske)',
                    r'\|[Ss]ource=Transferred from .*?; transferred to Commons by \[\[User:(.*?)\]\]']],
}

# Service images that don't have to be deleted and/or reported has a template inside them
# (you can let this param as None)
serviceTemplates = {
    '_default': None,
    'it': ['Template:Immagine di servizio'],
}

# Add your project (in alphabetical order) if you want that the bot start
project_inserted = [u'ar', u'commons', u'de', u'en', u'fa', u'ga', u'hu', u'it',
                    u'ja', u'ko', u'ta', u'zh']

# Ok, that's all. What is below, is the rest of code, now the code is fixed and it will run correctly in your project.
################################################################################
# <--------------------------- Change only above! ---------------------------> #
################################################################################

# Error Classes
class LogIsFull(pywikibot.Error):
    """An exception indicating that the log is full and the Bot cannot add
    other data to prevent Errors.

    """

class NothingFound(pywikibot.Error):
    """ An exception indicating that a regex has return [] instead of results.

    """

# Other common useful functions
def printWithTimeZone(message):
    """ Function to print the messages followed by the TimeZone encoded
    correctly.

    """
    if message[-1] != ' ':
        message = '%s ' % unicode(message)
    if locale.getlocale()[1]:
        time_zone = unicode(time.strftime(u"%d %b %Y %H:%M:%S (UTC)", time.gmtime()), locale.getlocale()[1])
    else:
        time_zone = unicode(time.strftime(u"%d %b %Y %H:%M:%S (UTC)", time.gmtime()))
    pywikibot.output(u"%s%s" % (message, time_zone))

class Global(object):
    # default environment settings
    # Command line configurable parameters
    repeat = True            # Restart after having check all the images?
    limit = 80               # How many images check?
    time_sleep = 30          # How many time sleep after the check?
    skip_number = 0          # How many images to skip before checking?
    waitTime = 0             # How many time sleep before the check?
    commonsActive = False    # Check if on commons there's an image with the same name?
    normal = False           # Check the new images or use another generator?
    urlUsed = False          # Use the url-related function instead of the new-pages generator
    regexGen = False         # Use the regex generator
    untagged = False         # Use the untagged generator
    duplicatesActive = False # Use the duplicate option
    duplicatesReport = False # Use the duplicate-report option
    sendemailActive = False  # Use the send-email
    logFullError = True      # Raise an error when the log is full


class main(checkimages.main):
#    def __init__(self, site, logFulNumber = 25000, sendemailActive = False,
#                 duplicatesReport = False, logFullError = True): pass
#    def setParameters(self, imageName, timestamp, uploader): pass
#    def report(self, newtext, image_to_report, notification=None, head=None,
#               notification2 = None, unver=True, commTalk=None, commImage=None): pass
#    def uploadBotChangeFunction(self, reportPageText, upBotArray): pass
#    def tag_image(self, put = True): pass
#    def put_mex_in_talk(self): pass
#    def untaggedGenerator(self, untaggedProject, limit): pass
#    def regexGenerator(self, regexp, textrun): pass
#    def loadHiddenTemplates(self): pass
#    def returnOlderTime(self, listGiven, timeListGiven): pass
#    def convert_to_url(self, page): pass
#    def countEdits(self, pagename, userlist): pass
#    def checkImageOnCommons(self): pass
#    def checkImageDuplicated(self, duplicates_rollback): pass
#    def report_image(self, image_to_report, rep_page = None, com = None, rep_text = None, addings = True, regex = None): pass
#    def takesettings(self): pass
#    def load_licenses(self): pass
#    def miniTemplateCheck(self, template): pass
#    def templateInList(self): pass
#    def smartDetection(self): pass
#    def load(self, raw): pass
#    def skipImages(self, skip_number, limit): pass
#    def wait(self, waitTime, generator, normal, limit): pass
#    def isTagged(self): pass
#    def findAdditionalProblems(self): pass

    def downloadImage(self):
        #print self.image
        self.image_filename = os.path.split(self.image.fileUrl())[-1]
        self.image_path     = os.path.join('cache', self.image_filename)
        if os.path.exists(self.image_path):
            return

#        f, data = self.site.getUrl(self.image.fileUrl(), no_hostname=True, back_response=True)
        # !!! CHEAP HACK TO GET IT WORKING -> NEEDS PATCH IN 'getUrl' upstream !!!
        # (prevent unicode encoding at end or allow to re-read in back_response)
        # (this will be useful for 'subster' also; merge several get modes there)
        req = urllib2.Request(self.image.fileUrl(), None, {})
        f = pywikibot.MyURLopener.open(req)
        data = f.read()

        f = open(self.image_path, 'wb')
        f.write( data )
        f.close()

    # LOOK ALSO AT: checkimages.main.checkStep
    # (and actegory scripts/bots too...)
    def checkStep(self):
        #print self.image_path
        pywikibot.output(self.image.title())

        # use explicit searches for classification
        for item in dir(self):
            if '_search' in item:
                (cat, result) = self.__class__.__dict__[item](self)
                #print cat, result
                if result:
                    pywikibot.output( u'   {{%s}} found %i time(s)'
                                      % (cat, len(result)) )

        # use general classification
        pywikibot.output( u'   {{%s}} ...' % self._classify()[0] )

    # Category:Unidentified people
    def _searchPeople(self):
        result = []
        try:
            result = self._CVdetectObjects()
        except IOError:
            pywikibot.output(u'WARNING: unknown file type')
        #print self.image, '\n   ', result

        return (u'Category:Unidentified people', result)

    def _CVdetectObjects(self):
        """Converts an image to grayscale and prints the locations of any
           faces found"""
        # http://python.pastebin.com/m76db1d6b
        # http://creatingwithcode.com/howto/face-detection-in-static-images-with-python/
        # http://opencv.willowgarage.com/documentation/python/objdetect_cascade_classification.html
        # http://opencv.willowgarage.com/wiki/FaceDetection
        # http://blog.jozilla.net/2008/06/27/fun-with-python-opencv-and-face-detection/
        # http://www.cognotics.com/opencv/servo_2007_series/part_4/index.html
        
        #from opencv.cv import *
        #from opencv.highgui import *
        import cv

        image = cv.LoadImage(self.image_path)

        grayscale = cv.CreateImage((image.width, image.height), 8, 1)
        cv.CvtColor(image, grayscale, cv.CV_BGR2GRAY)

        storage = cv.CreateMemStorage(0)
        #storage = cv.CreateMemStorage()
        #cv.ClearMemStorage(storage)
        cv.EqualizeHist(grayscale, grayscale)
        #cascade = cv.LoadHaarClassifierCascade(
        cascade = cv.Load(
          '/usr/share/opencv/haarcascades/haarcascade_frontalface_default.xml',
          #(1,1))
          )
        faces = cv.HaarDetectObjects(grayscale, cascade, storage, 1.2, 2,
                                   cv.CV_HAAR_DO_CANNY_PRUNING, (50,50))

        if faces:
            for f in faces:
                (x, y, width, height) = f[0]
                #print("[(%d,%d) -> (%d,%d)]" % (f.x, f.y, f.x+f.width, f.y+f.height))
                (x1, y1, x2, y2) = (x, y, x+width, y+height)
                #print("[(%d,%d) -> (%d,%d)]" % (x1, y1, x2, y2))
                self._drawRect(x1,y1,x2,y2) #call to a python pil

        return faces

    def _drawRect(self, x1,y1,x2,y2): #function to modify the img
        import Image, ImageDraw
        image_path_new = os.path.join('cache', '0_DETECTED_' + self.image_filename)
        im = Image.open(self.image_path)
        draw = ImageDraw.Draw(im)
        draw.rectangle([x1,y1,x2,y2], outline=(255,0,0))
        im.save(image_path_new)

    # Category:Unidentified people
    # Category:Unidentified maps
    # Category:Unidentified flags
    # Category:Unidentified plants
    # Category:Unidentified coats of arms
    # Category:Unidentified buildings
    # Category:Unidentified trains
    # Category:Unidentified automobiles
    # Category:Unidentified buses
    def _classify(self):
        return self._CVclassifyObjects()

    def _CVclassifyObjects(self):
        """Uses the 'The Bag of Words model' for classification"""
        # http://app-solut.com/blog/2011/07/the-bag-of-words-model-in-opencv-2-2/
        # http://app-solut.com/blog/2011/07/using-the-normal-bayes-classifier-for-image-categorization-in-opencv/
        # http://authors.library.caltech.edu/7694/
        # http://www.vision.caltech.edu/Image_Datasets/Caltech256/
        
        # https://code.ros.org/trac/opencv/browser/trunk/opencv/samples/cpp/bagofwords_classification.cpp?rev=3714
        # stand-alone (in shell) for training e.g. with:
        #   BoWclassify /data/toolserver/_tests/VOCdevkit/VOC2007 /data/toolserver/_tests/VOCdevkit/result FAST SURF BruteForce
        import opencv
        #result = opencv.BoWclassify.main(0, '', '', '', '', '')
        result = opencv.BoWclassify.main(6, 
                                         '/data/toolserver/_tests/VOCdevkit/VOC2007', 
                                         '/data/toolserver/_tests/VOCdevkit/result', 
                                         'FAST', 
                                         'SURF', 
                                         'BruteForce')
        print result
        raise

        return (u'Category:?', result)

gbv = Global()

def checkbot():
    """ Main function """
    # Command line configurable parameters
    repeat = True # Restart after having check all the images?
    limit = 80 # How many images check?
    time_sleep = 30 # How many time sleep after the check?
    skip_number = 0 # How many images to skip before checking?
    waitTime = 0 # How many time sleep before the check?
    commonsActive = False # Check if on commons there's an image with the same name?
    normal = False # Check the new images or use another generator?
    urlUsed = False # Use the url-related function instead of the new-pages generator
    regexGen = False # Use the regex generator
    untagged = False # Use the untagged generator
    duplicatesActive = False # Use the duplicate option
    duplicatesReport = False # Use the duplicate-report option
    sendemailActive = False # Use the send-email
    logFullError = True # Raise an error when the log is full

    # emulate: 'python checkimages_content.py -limit:5 -break -lang:en'
    sys.argv += ['-limit:20', '-break', '-lang:en']
    print "http://commons.wikimedia.org/wiki/User:Multichill/Using_OpenCV_to_categorize_files"

    # Here below there are the parameters.
    for arg in pywikibot.handleArgs():
        if arg.startswith('-limit'):
            if len(arg) == 7:
                limit = int(pywikibot.input(u'How many files do you want to check?'))
            else:
                limit = int(arg[7:])
        if arg.startswith('-time'):
            if len(arg) == 5:
                time_sleep = int(pywikibot.input(u'How many seconds do you want runs to be apart?'))
            else:
                time_sleep = int(arg[6:])
        elif arg == '-break':
            repeat = False
        elif arg == '-nologerror':
            logFullError = False
        elif arg == '-commons':
            commonsActive = True
        elif arg.startswith('-duplicates'):
            duplicatesActive = True
            if len(arg) == 11:
                duplicates_rollback = 1
            elif len(arg) > 11:
                duplicates_rollback = int(arg[12:])
        elif arg == '-duplicatereport':
            duplicatesReport = True
        elif arg == '-sendemail':
            sendemailActive = True
        elif arg.startswith('-skip'):
            if len(arg) == 5:
                skip = True
                skip_number = int(pywikibot.input(u'How many files do you want to skip?'))
            elif len(arg) > 5:
                skip = True
                skip_number = int(arg[6:])
        elif arg.startswith('-wait'):
            if len(arg) == 5:
                wait = True
                waitTime = int(pywikibot.input(u'How many time do you want to wait before checking the files?'))
            elif len(arg) > 5:
                wait = True
                waitTime = int(arg[6:])
        elif arg.startswith('-start'):
            if len(arg) == 6:
                firstPageTitle = pywikibot.input(u'From witch page do you want to start?')
            elif len(arg) > 6:
                firstPageTitle = arg[7:]
            firstPageTitle = firstPageTitle.split(":")[1:]
            generator = pywikibot.getSite().allpages(start=firstPageTitle, namespace=6)
            repeat = False
        elif arg.startswith('-page'):
            if len(arg) == 5:
                regexPageName = str(pywikibot.input(u'Which page do you want to use for the regex?'))
            elif len(arg) > 5:
                regexPageName = str(arg[6:])
            repeat = False
            regexGen = True
        elif arg.startswith('-url'):
            if len(arg) == 4:
                regexPageUrl = str(pywikibot.input(u'Which url do you want to use for the regex?'))
            elif len(arg) > 4:
                regexPageUrl = str(arg[5:])
            urlUsed = True
            repeat = False
            regexGen = True
        elif arg.startswith('-regex'):
            if len(arg) == 6:
                regexpToUse = str(pywikibot.input(u'Which regex do you want to use?'))
            elif len(arg) > 6:
                regexpToUse = str(arg[7:])
            generator = 'regex'
            repeat = False
        elif arg.startswith('-cat'):
            if len(arg) == 4:
                catName = str(pywikibot.input(u'In which category do I work?'))
            elif len(arg) > 4:
                catName = str(arg[5:])
            catSelected = catlib.Category(pywikibot.getSite(), 'Category:%s' % catName)
            generator = pagegenerators.CategorizedPageGenerator(catSelected)
            repeat = False
        elif arg.startswith('-ref'):
            if len(arg) == 4:
                refName = str(pywikibot.input(u'The references of what page should I parse?'))
            elif len(arg) > 4:
                refName = str(arg[5:])
            generator = pagegenerators.ReferringPageGenerator(pywikibot.Page(pywikibot.getSite(), refName))
            repeat = False
        elif arg.startswith('-untagged'):
            untagged = True
            if len(arg) == 9:
                projectUntagged = str(pywikibot.input(u'In which project should I work?'))
            elif len(arg) > 9:
                projectUntagged = str(arg[10:])

    # Understand if the generator it's the default or not.
    try:
        generator
    except NameError:
        normal = True

    # Define the site.
    site = pywikibot.getSite()

    # Block of text to translate the parameters set above.
    image_old_namespace = u"%s:" % site.image_namespace()
    image_namespace = u"File:"

    # If the images to skip are 0, set the skip variable to False (the same for the wait time)
    if skip_number == 0:
        skip = False
    if waitTime == 0:
        wait = False

    # A little block-statement to ensure that the bot will not start with en-parameters
    if site.lang not in project_inserted:
        pywikibot.output(u"Your project is not supported by this script. You have to edit the script and add it!")
        return

    # Reading the log of the new images if another generator is not given.
    if normal == True:
        if limit == 1:
            pywikibot.output(u"Retrieving the latest file for checking...")
        else:
            pywikibot.output(u"Retrieving the latest %d files for checking..." % limit)
    # Main Loop
    while 1:
        # Defing the Main Class.
        mainClass = main(site, sendemailActive = sendemailActive,
                         duplicatesReport = duplicatesReport, logFullError = logFullError)
        # Untagged is True? Let's take that generator
        if untagged == True:
            generator =  mainClass.untaggedGenerator(projectUntagged, limit)
            normal = False # Ensure that normal is False
        # Normal True? Take the default generator
        if normal == True:
            generator = site.newimages(number = limit)
        # if urlUsed and regexGen, get the source for the generator
        if urlUsed == True and regexGen == True:
            textRegex = site.getUrl(regexPageUrl, no_hostname = True)
        # Not an url but a wiki page as "source" for the regex
        elif regexGen == True:
            pageRegex = pywikibot.Page(site, regexPageName)
            try:
                textRegex = pageRegex.get()
            except pywikibot.NoPage:
                pywikibot.output(u"%s doesn't exist!" % pageRegex.title())
                textRegex = '' # No source, so the bot will quit later.
        # If generator is the regex' one, use your own Generator using an url or page and a regex.
        if generator == 'regex' and regexGen == True:
            generator = mainClass.regexGenerator(regexpToUse, textRegex)
        # Ok, We (should) have a generator, so let's go on.
        # Take the additional settings for the Project
        mainClass.takesettings()
        # Not the main, but the most important loop.
        #parsed = False
        if wait:
            # Let's sleep...
            generator = mainClass.wait(waitTime, generator, normal, limit)
        for image in generator:
            # When you've a lot of image to skip before working use this workaround, otherwise
            # let this commented, thanks. [ decoment also parsed = False if you want to use it
            #
            #if image.title() != u'File:Nytlogo379x64.gif' and not parsed:
            #    pywikibot.output(u"%s already parsed." % image.title())
            #    continue
            #else:
            #    parsed = True
            # If the generator returns something that is not an image, simply skip it.
            if normal == False and regexGen == False:
                if image_namespace.lower() not in image.title().lower() and \
                image_old_namespace.lower() not in image.title().lower() and 'file:' not in image.title().lower():
                    pywikibot.output(u'%s seems not an file, skip it...' % image.title())
                    continue
            if normal:
                imageData = image
                image = imageData[0]
                #20100511133318L --- 15:33, 11 mag 2010 e 18 sec
                #b = str(imageData[1]) # use b as variable to make smaller the timestamp-formula used below..
                # fixing the timestamp to the format that we normally use..
                timestamp = imageData[1]#"%s-%s-%sT%s:%s:%sZ" % (b[0:4], b[4:6], b[6:8], b[8:10], b[10:12], b[12:14])
                uploader = imageData[2]
                comment = imageData[3] # useless, in reality..
            else:
                timestamp = None
                uploader = None
                comment = None # useless, also this, let it here for further developments
            try:
                imageName = image.title().split(image_namespace)[1] # Deleting the namespace (useless here)
            except IndexError:# Namespace image not found, that's not an image! Let's skip...
                try:
                    imageName = image.title().split(image_old_namespace)[1]
                except IndexError:
                    pywikibot.output(u"%s is not a file, skipping..." % image.title())
                    continue
            mainClass.setParameters(imageName, timestamp, uploader) # Setting the image for the main class
            mainClass.downloadImage()
            # Skip block
            if skip == True:
                skip = mainClass.skipImages(skip_number, limit)
                if skip == True:
                    continue
            # Check on commons if there's already an image with the same name
            if commonsActive == True and site.family.name != "commons":
                response = mainClass.checkImageOnCommons()
                if response == False:
                    continue
            # Check if there are duplicates of the image on the project selected
            if duplicatesActive == True:
                response2 = mainClass.checkImageDuplicated(duplicates_rollback)
                if response2 == False:
                    continue
            resultCheck = mainClass.checkStep()
            if resultCheck:
                continue
    # A little block to perform the repeat or to break.
        if repeat == True:
            printWithTimeZone(u"Waiting for %s seconds," % time_sleep)
            time.sleep(time_sleep)
        else:
            pywikibot.output(u"\t\t\t>> STOP! <<")
            break # Exit


# Main loop will take all the (name of the) images and then i'll check them.
if __name__ == "__main__":
    old = datetime.datetime.strptime(str(datetime.datetime.utcnow()).split('.')[0], "%Y-%m-%d %H:%M:%S") #timezones are UTC
    try:
        checkbot()
    finally:
        final = datetime.datetime.strptime(str(datetime.datetime.utcnow()).split('.')[0], "%Y-%m-%d %H:%M:%S") #timezones are UTC
        delta = final - old
        secs_of_diff = delta.seconds
        pywikibot.output("Execution time: %s" % secs_of_diff)
        pywikibot.stopme()
